#*****************************************************************************
#* File: lua/modules/ui/game/commandmode.lua
#* Author: Chris Blackwell
#* Summary: Manages the current command mode, which determines what action
#* the mouse will take when next clicked in the world
#*
#* Copyright  2005 Gas Powered Games, Inc.  All rights reserved.
#*****************************************************************************
local USC = import('/lua/user/UserScriptCommand.lua')
local Dragger = import('/lua/maui/dragger.lua').Dragger
local Construction = import('/lua/ui/game/construction.lua')
local UIMain = import('/lua/ui/uimain.lua')
local Orders = import('/lua/ui/game/orders.lua')
local Pri = import('/mods/TargetPriorityChanger/hook/lua/ui/game/PriorityList.lua')
local USC = import('/lua/user/UserScriptCommand.lua')

local commandMode = false
local modeData = false
local issuedOneCommand = false
local CommandModeActive = false
local startBehaviors = {}
local endBehaviors = {}
local SavedPriorities = {}

function OnCommandModeBeat()
	if issuedOneCommand and not IsKeyDown('Shift') then
		EndCommandMode(true,true)
	end
end

import('/lua/ui/game/gamemain.lua').AddBeatFunction(OnCommandModeBeat)

-- behaviors are functions that take a single string parameter, the commandMode (or false if none)
function AddStartBehavior(behavior)
    table.insert(startBehaviors, behavior)
end

function AddEndBehavior(behavior)
    table.insert(endBehaviors, behavior)
end

function StartCommandMode(newCommandMode, data)
	#LOG('startCommandMode: ', newCommandMode)
    if commandMode then
        EndCommandMode(false)
    end
    commandMode = newCommandMode
    modeData = data
    for i,v in startBehaviors do
        v(commandMode, modeData)
    end
end

function GetCommandMode()
    return {commandMode, modeData}
end

function EndCommandMode(clearModifier,ShiftEnd)
	if ShiftEnd then
		if modeData.name == 'RULEUCC_Attack' then
			SimCallback({Func="DataSet",
						 Args={
						 		CommandMode = true,
						 		PriorityList = SavedPriorities,
						 		Shift = true,
							  }
						},true
					   )
		end
	end
    for i,v in endBehaviors do
        v(commandMode, modeData)
    end
	
    commandMode = false
    modeData = false
    SavedPriorities = {}
	issuedOneCommand = false
	issuedTwoCommand = false
	
	if clearModifier then
   	    Construction.ClearWhatIfModifier()
   	end
   	

end

commandFeedbackResource = {
	Move = { '/meshes/game/Move_lod0.scm', '/meshes/game/Move_albedo.dds' },
	Attack = { '/meshes/game/Attack_lod0.scm', '/meshes/game/Attack_albedo.dds' },
	Guard = { '/meshes/game/Assist_lod0.scm', '/meshes/game/Assist_albedo.dds' },
	Capture = { '/meshes/game/Capture_lod0.scm', '/meshes/game/Capture_albedo.dds' },
	Ferry = { '/meshes/game/Ferry_lod0.scm', '/meshes/game/Ferry_albedo.dds' },
	Nuke = { '/meshes/game/Launch_Missile02_lod0.scm', '/meshes/game/Launch_Missile02_albedo.dds' },
	Tactical = { '/meshes/game/Launch_Missile01_lod0.scm', '/meshes/game/Launch_Missile01_albedo.dds' },
	TransportLoadUnits = { '/meshes/game/Load_lod0.scm', '/meshes/game/Load_albedo.dds' },
	TransportReverseLoadUnits = { '/meshes/game/Load_lod0.scm', '/meshes/game/Load_albedo.dds' },
	TransportUnloadUnits = { '/meshes/game/Unload_lod0.scm', '/meshes/game/Unload_albedo.dds' },	
	OverCharge = { '/meshes/game/Overcharge_lod0.scm', '/meshes/game/Overcharge_albedo.dds' },
	Patrol = { '/meshes/game/Patrol_lod0.scm', '/meshes/game/Patrol_albedo.dds' },
	Reclaim = { '/meshes/game/Reclaim_lod0.scm', '/meshes/game/Reclaim_albedo.dds' },
	Repair = { '/meshes/game/Repair_lod0.scm', '/meshes/game/Repair_albedo.dds' },
	Sacrifice = { '/meshes/game/Sacrifice_lod0.scm', '/meshes/game/Sacrifice_albedo.dds' },
	Teleport = { '/meshes/game/Teleport_lod0.scm', '/meshes/game/Teleport_albedo.dds' },	
}

function AddCommandFeedbackByType(pos, type)

	if commandFeedbackResource[type] == nil then
		return false;
	else
		#LOG(repr(type))
		AddCommandFeedbackBlip({
					Position = pos, 
					MeshName = commandFeedbackResource[type][1],
					TextureName = commandFeedbackResource[type][2],
					ShaderName = 'CommandFeedback',
					UniformScale = 0.125,
				}, 0.7)
	end
		
	return true;
end
	

function OnCommandIssued(command)
	if IsKeyDown('Shift') then
		table.insert(SavedPriorities, command.Target.EntityId)
	end
    if not command.Clear then
		issuedOneCommand = true
	else	
		if command.CommandType == 'Attack' then
			
			SimCallback({Func="DataSet",
						 Args={
						 		CommandMode = true,
						 		PriorityList = {command.Target.EntityId},
						 		
							  }
						},true
					   )
		end
		EndCommandMode(true)
	end

		
	if command.CommandType == 'BuildMobile' then				
		AddCommandFeedbackBlip({
			Position = command.Target.Position, 
			BlueprintID = command.Blueprint,			
			TextureName = '/meshes/game/flag02d_albedo.dds',
			ShaderName = 'CommandFeedback',
			UniformScale = 1,
		}, 0.7)	
	else	
	
		if AddCommandFeedbackByType(command.Target.Position, command.CommandType) == false then	
			AddCommandFeedbackBlip({
				Position = command.Target.Position, 
				MeshName = '/meshes/game/flag02d_lod0.scm',
				TextureName = '/meshes/game/flag02d_albedo.dds',
				ShaderName = 'CommandFeedback',
				UniformScale = 0.5,
			}, 0.7)		
			
			AddCommandFeedbackBlip({
				Position = command.Target.Position, 
				MeshName = '/meshes/game/crosshair02d_lod0.scm',
				TextureName = '/meshes/game/crosshair02d_albedo.dds',
				ShaderName = 'CommandFeedback2',
				UniformScale = 0.5,
			}, 0.75)		
		end		
	end
end
